'''
Created on 9 mars 2021

@author: laborie
'''
from django import forms
from mri_users.models import NewUser, User, WaitingValidationMails, MailsModels
from django.contrib.auth import password_validation
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from mri_users.helpers import change_ldap_user_data, change_user_passwd,\
    verify_password
from MRIOutils import settings

class RegisterForm(forms.ModelForm):
    """
    Formulaire de création de compte
    """
    error_messages = {
        'password_mismatch': _('The two password fields didn’t match.'),
        'email_exists': _('An account with this email already exists')
    }
    email = forms.EmailField()
    password1 = forms.CharField(
        label=_("Password"),
        strip=False,
        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
        strip=False,
        help_text=_("Enter the same password as before, for verification."),
    )

    class Meta:
        model = NewUser
        fields = ("first_name", "last_name", "email")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['first_name'].required = True
        self.fields['last_name'].required = True

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )
        return password2
    
    def clean_email(self):
        email = self.cleaned_data.get("email")
        if NewUser.objects.filter(email=email).exists() or User.objects.filter(email=email).exists():
            raise ValidationError(
                self.error_messages['email_exists'],
                code='Email exists in database',
            )
        return email

    def _post_clean(self):
        super()._post_clean()
        # Validate the password after self.instance is updated with form data
        # by super().
        password = self.cleaned_data.get('password2')
        if password:
            try:
                password_validation.validate_password(password, self.instance)
            except ValidationError as error:
                self.add_error('password2', error)

    def save(self, commit=True):
        user = super().save(commit=False)
        user.password = settings.ENCRYPT_ALGO.hash(self.cleaned_data.get("password1"))
        if commit:
            user.save()
        return user

class SetPasswordForm(forms.Form):
    """
    Forme de changement de mot de passe sans l'ancien
    """
    error_messages = {
        'password_mismatch': _('The two password fields didn’t match.'),
    }
    new_password1 = forms.CharField(
        label=_("New password"),
        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
        strip=False,
        help_text=password_validation.password_validators_help_text_html(),
    )
    new_password2 = forms.CharField(
        label=_("New password confirmation"),
        strip=False,
        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
    )

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super().__init__(*args, **kwargs)

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise ValidationError(
                    self.error_messages['password_mismatch'],
                    code='password_mismatch',
                )
        password_validation.validate_password(password2, self.user)
        return password2

    def save(self, commit=True):
        password = self.cleaned_data["new_password1"]
        change_user_passwd(self.user, password, changed_with_link=True)


class PasswordChangeForm(SetPasswordForm):
    """
    Forme de changement de mot de passe en validant l'ancien
    """
    error_messages = {
        **SetPasswordForm.error_messages,
        'password_incorrect': _("Your old password was entered incorrectly. Please enter it again."),
    }
    old_password = forms.CharField(
        label=_("Old password"),
        strip=False,
        widget=forms.PasswordInput(attrs={'autocomplete': 'current-password', 'autofocus': True}),
    )

    field_order = ['old_password', 'new_password1', 'new_password2']

    def clean_old_password(self):
        """
        Valide l'ancien mdp.
        """
        old_password = self.cleaned_data["old_password"]
        if not verify_password(self.user, old_password):
            raise ValidationError(
                self.error_messages['password_incorrect'],
                code='password_incorrect',
            )
        return old_password


class ChangePasswdLinkForm(forms.Form):
    """
    Forme de demande de lien de reset mdp
    """
    email = forms.EmailField()


class SearchUser(forms.Form):
    """
    Simple form pour la recherche d'utilisateur
    """
    search = forms.CharField(label=_('Search'), max_length=20,min_length=3)


class ManageMyAccountForm(forms.ModelForm):
    """
    Forme de changement d'info pour les utilisateurs
    """
    error_messages = {
        'email_exists': _('An account with this email already exists')
    }

    def __init__(self, *args, **kwargs):
        self.request=kwargs.pop('request')
        super(ManageMyAccountForm, self).__init__(*args, **kwargs)
        self.fields['username'].disabled = True
        self.fields['username'].help_text = ''
        self.fields['first_name'].required = True
        self.fields['last_name'].required = True
        self.fields['email'].required = True
        if hasattr(self.instance, 'waitingvalidationmails'):
            self.fields['email'].disabled = True

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']

    def clean_email(self):
        if 'email' in self.changed_data:
            email = self.cleaned_data.get("email")
            if NewUser.objects.filter(email=email).exists() or User.objects.filter(email=email).exists():
                raise ValidationError(
                    self.error_messages['email_exists'],
                    code='Email exists in database',
                )
            return email

    def save(self, commit=True):
        attr_field = {
            'email': 'mail',
            'first_name': 'givenName',
            'last_name': 'sn',
            'account_type': 'accountType'}
        old = {}
        new = {}
        for field in self.changed_data:
            if field == 'email' and self.request.user.account_type != 'administrator':
                WaitingValidationMails(user=self.instance, new_email=self.cleaned_data[field]).save()
                continue
            old[attr_field[field]] = [self.initial[field].encode('utf-8')]
            new[attr_field[field]] = [self.cleaned_data[field].encode('utf-8')]
        if len(new):
            change_ldap_user_data(self.instance, old, new)


class ManageAccountForm(ManageMyAccountForm):
    """
    Forme de changement d'info pour les administrateurs
    """
    error_messages = {
        'email_exists_new_user_valid_mail': _('A registration is pending validation with this email'),
        'email_exists_new_user_not_valid_mail': _('A registration with this email is pending confirmation of the email by the user'),
        'email_exists_user': _('An account with this email already exists')
    }

    class Meta(ManageMyAccountForm.Meta):
        fields = ManageMyAccountForm.Meta.fields + ['account_type']

    def clean_email(self):
        if 'email' in self.changed_data:
            email = self.cleaned_data.get("email")
            if NewUser.objects.filter(email=email).exists():
                if NewUser.objects.filter(email=email, email_confirmed=False).exists():
                    raise ValidationError(
                        self.error_messages['email_exists_new_user_not_valid_mail'],
                        code='Email exists in newuser with unconfirmed mail',
                    )
                else:
                    raise ValidationError(
                        self.error_messages['email_exists_new_user_valid_mail'],
                        code='Email exists in newuser',
                    )
            elif User.objects.filter(email=email).exists():
                raise ValidationError(
                    self.error_messages['email_exists_user'],
                    code='Email exists in database',
                )
            return email


class RefuseRegister(forms.Form):
    """
    Forme pour donner une raison au refus (envoyé par mail)
    """
    raison = forms.CharField(max_length=150, min_length=10)


class LockAccountForm(forms.Form):
    """
    Forme pour donner une raison au verrouillage d'un compte (envoyé par mail)
    """
    raison = forms.CharField(max_length=150, min_length=10)


class MailsModelsForm(forms.ModelForm):
    """
    Formulaire pour page edition des mails
    """
    class Meta:
        model = MailsModels
        fields = ("title", "body_fr", "body_en")

